
/* AXP Mapped Box by Tom Hudson, Version 1.0 */
/* For Autodesk 3D Studio V2.0 or 2.01 or greater.     */
/* This program takes any box with a texture map applied to it */
/* (with any arbitrary mapping coordinates also applied) and */
/* tiles the texture map on each of the boxes six sides.  */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "axp.h"
#include "trig.h"

#define Rand01() ((float)(rand() & 0x7fff) / 32767.0)

/* Variable definitions */

#define REPEAT 1
#define DELTAU 2
#define DELTAV 3

/************************/

/* Dialog description */

DlgEntry cdialog[]={
	0,"TITLE=\"Autodesk 3D Studio Mapped Box Generator\"",
	0,"TITLE=\"Version 1.0\"",
	0,"TITLE=\"Copyright 1993 Tom Hudson.  From Yost Group, Inc.\"",
	0,"TITLE=\"This is a Public Domain IPAS Routine.  Not for Resale\"",
	0,"TITLE=\"\"",
	REPEAT,"SLIDER=\"Tiling:\",1,100,30",
 	DELTAU,"FLOAT=\"Delta U:\",8",
 	DELTAV,"-FLOAT=\"Delta V:\",8",
	0,"TITLE=\"\"",
	0,NULL
	};

/* Version test value */

#define VERSION 0x9192
	
typedef struct {
	ulong version;
	int repeat;
	float deltau,deltav;
	} State;

/* the "state" struct MUST start with a "ulong" which is the version#,
	to prevent using data from old versions of this program.
	This verification is performed automatically. */

static State init_state = { VERSION,1,0.01,0.01 };
static State state = { VERSION,1,0.01,0.01 };

/* Variables */

#define LL (FC_ABLINE | FC_BCLINE)
#define UR (FC_BCLINE | FC_CALINE)

static VData vdata[24],vsource[24]=
{
0,0,0,0,1,
1,0,0,0,0,
2,0,0,1,0,
3,0,0,1,1,
3,0,0,0,1,
2,0,0,0,0,
4,0,0,1,0,
5,0,0,1,1,
5,0,0,0,1,
4,0,0,0,0,
6,0,0,1,0,
7,0,0,1,1,
7,0,0,0,1,
6,0,0,0,0,
1,0,0,1,0,
0,0,0,1,1,
7,0,0,0,1,
0,0,0,0,0,
3,0,0,1,0,
5,0,0,1,1,
1,0,0,0,1,
6,0,0,0,0,
4,0,0,1,0,
2,0,0,1,1
};

static FData fdata[12]=
{
0,1,2,1,LL,255,
0,2,3,1,UR,255,
4,5,6,1,LL,255,
4,6,7,1,UR,255,
8,9,10,1,LL,255,
8,10,11,1,UR,255,
12,13,14,1,LL,255,
12,14,15,1,UR,255,
16,17,18,1,LL,255,
16,18,19,1,UR,255,
20,21,22,1,LL,255,
20,22,23,1,UR,255
};

static int frame,particle,overts,burstmode;
static float sizefac,partx,party,partz,px,py,pz,radius,orbit_time,position;
static float sinfac,cosfac,xfac,yfac,zfac,fchaos,forder,taper,ybasis,ylimit;
static int dialog_ix,verts,faces,first,ix,onedges,offedges,edges,totlights,edge;
static int vertex,face;
static float minx,miny,minz,maxx,maxy,maxz,dx,dy,dz;
static unsigned char material;

static float *xvals[8]=
{&minx,&minx,&maxx,&maxx,&maxx,&maxx,&minx,&minx};
static float *yvals[8]=
{&miny,&miny,&miny,&miny,&maxy,&maxy,&maxy,&maxy};
static float *zvals[8]=
{&maxz,&minz,&minz,&maxz,&minz,&maxz,&minz,&maxz};

/*----------------------------------------------------------------*/

void ClientSetStateVar(int id, void *ptr) {
	OVL o;
	ulong *ul;

	ul=(ulong *)ptr;
	o.ul = *ul;
	switch(id) {
		case REPEAT: state.repeat=o.i; break;
		case DELTAU: state.deltau=o.f; break;
		case DELTAV: state.deltav=o.f; break;
		}
	}

ulong ClientGetStateVar(int id) {
	OVL o;
	switch(id) {
		case REPEAT: o.i=state.repeat; break;
		case DELTAU: o.f=state.deltau; break;
		case DELTAV: o.f=state.deltav; break;
		}
	return(o.ul);
	}

int ClientVarSize(int id)
	{
	switch(id)
		{
		default:
			return(1);
		}
	}

char  *ClientGetState(int *size) {
	*size = sizeof(State);
	return((char *)&state);
	}

void ClientResetState() {	
	state = init_state;	
	}

/* Client process starts here */

void ClientStartup(EXPbuf *buf) {
#define ob buf->data.object
	overts=ob.verts;	/* Get existing object's count for minmaxing */
	frame=ob.curfield;
#undef ob
	burstmode=(studio_version()!=200) ? 1:0;

	if(burstmode)	/* Version 2.01 way... */
		{
		/* Find object's extents */
		buf->opcode=EXP_OBJ_EXTENTS;	/* Studio does this */
		buf->usercode=0x0220;		/* Do this next time */
		buf->status=1;
		}
	else	/* Do it the Version 2.0 way... */
		{
		first=1;
		ix=0;

#define cv buf->data.vert
		cv.vertex=ix;
		buf->opcode=EXP_GET_VERTEX;	/* Studio does this */
		buf->usercode=0x0200;		/* Do this next time */
		buf->status=1;
#undef cv
		}
	}

/* User routines -- use user codes to process your data */

void ClientUserCode(EXPbuf *buf)
	{
	switch(buf->usercode)
		{
		case 0x0200:	/* Extents the old (2.0) way */
#define cv buf->data.vert
			if(first)
				{
				minx=maxx=cv.x;
				miny=maxy=cv.y;
				minz=maxz=cv.z;
				first=0;
				}
			else
				{
				if(cv.x<minx)
					minx=cv.x;
				else
				if(cv.x>maxx)
					maxx=cv.x;
				if(cv.y<miny)
					miny=cv.y;
				else
				if(cv.y>maxy)
					maxy=cv.y;
				if(cv.z<minz)
					minz=cv.z;
				else
				if(cv.z>maxz)
					maxz=cv.z;
				}
			ix++;
			if(ix<overts)
				{
				cv.vertex=ix;
				buf->opcode=EXP_GET_VERTEX;	/* Studio does this */
				buf->usercode=0x0200;		/* Do this next time */
				buf->status=1;
				break;
				}
#undef cv
			goto calc_deltas;

		case 0x0220:	/* Extents the new (2.01) way */
#define ex buf->data.exts
			minx=ex.minx;
			maxx=ex.maxx;
			miny=ex.miny;
			maxy=ex.maxy;
			minz=ex.minz;
			maxz=ex.maxz;
#undef ex
			calc_deltas:
			dx=maxx-minx;
			dy=maxy-miny;
			dz=maxz-minz;

			/* Find object material */

			buf->opcode=EXP_GET_FACE;
			buf->usercode=0x0230;
#define cf buf->data.face
			cf.face=0;
			break;
		case 0x0230:
			material=cf.material;
#undef cf
			/* Create the necessary number of verts & faces */

			buf->opcode=EXP_READY_OBJ;
			buf->usercode=0x0240;
			buf->status=1;
#define ob buf->data.object
			ob.verts=24;
			ob.tverts=24;
			ob.faces=12;
#undef ob
			break;
		case 0x0240:
			vertex=face=0;
			if(buf->status==0)
				{
				terminate:
				buf->opcode=buf->usercode=EXP_TERMINATE;
				break;
				}

			/* Load vertex list */
			for(ix=0; ix<24; ++ix)
				{
				int index=(int)vsource[ix].x;
				vdata[ix].x= *(xvals[index]);
				vdata[ix].y= *(yvals[index]);
				vdata[ix].z= *(zvals[index]);
				vdata[ix].u=vsource[ix].u*(float)state.repeat+(float)frame*state.deltau;
				vdata[ix].v=vsource[ix].v*(float)state.repeat+(float)frame*state.deltav;
				}

			/* Now create the appropriate vertices for this particle */

			if(burstmode)
				{
				buf->opcode=EXP_PUT_VLIST;
				buf->usercode=0x0270;
#define cv buf->data.vlist
				cv.start=0;
				cv.count=24;
				cv.data=(_far VData *)vdata;
#undef cv
				break;
				}

			old_vloop:
#define cv buf->data.vert
			buf->opcode=EXP_PUT_VERTEX;
			buf->usercode=0x0260;
			cv.x=vdata[vertex].x;
			cv.y=vdata[vertex].y;
			cv.z=vdata[vertex].z;
			cv.u=vdata[vertex].u;
			cv.v=vdata[vertex].v;
			cv.vertex=vertex++;
			break;

		case 0x0260:
			if(vertex<24)
				goto old_vloop;
#undef cv
			/* Now add the appropriate faces */

		case 0x0270:
			pvert_done:
			ix=face=0;

			face_loop:
			if(burstmode)
				{
				for(ix=0; ix<12; ++ix)
					fdata[ix].material=material;

				buf->opcode=EXP_PUT_FLIST;
				buf->usercode=0x0350;
#define cf buf->data.flist
				cf.start=0;
				cf.count=12;
				cf.data=(_far FData *)fdata;
#undef cf
				break;		
				}

			old_floop:
#define cf buf->data.face
			buf->opcode=EXP_PUT_FACE;
			buf->usercode=0x0305;
			cf.a=fdata[face].a;
			cf.b=fdata[face].b;
			cf.c=fdata[face].c;
			cf.flags=fdata[face].flags;
			cf.sm_group=fdata[face].sm_group;
			cf.material=material;
			cf.face=face++;
			break;

		case 0x0305:
			if(face<12)
				goto old_floop;

		case 0x0350:
			createit:
			buf->opcode=EXP_CREATE_OBJ;
			buf->usercode=EXP_TERMINATE;
			buf->status=1;
			break;

		default:
		error("Invalid user opcode",NULL,NULL,NULL,NULL);
		buf->opcode=buf->usercode=EXP_TERMINATE;
		buf->status=0;
		}
	}

void ClientTerminate(void) { 
	/* free any data structures, etc. */
	}

DlgEntry *ClientDialog(int n) {	
	return(&cdialog[n]); 
	}

char *ClientName(void)
	{
	return("MAPBOX.AXP");		/* Put your process name here */
	}
